var RetractionFollowUpForm = function()
{
	var rf = {};
	var patientInfo;
	var caseIsValid = false;
	var labSentDateIsValid = false;
	var labReceivedDateIsValid = false;
	var bloodSentDateIsValid = false;
	var bloodReceivedDateIsValid = false;
	var frozenSentDateIsValid = false;
	var frozenReceivedDateIsValid = false;
	var forwardedSentDateIsValid = false;
	var debug = 0;
	
	rf.initPage = function()
	{
		caseIsValid = false;
		labSentDateIsValid = false;
		labReceivedDateIsValid = false;
		bloodSentDateIsValid = false;
		bloodReceivedDateIsValid = false;
		frozenSentDateIsValid = false;
		frozenReceivedDateIsValid = false;
		forwardedSentDateIsValid = false;
	
		previewTitle = Doc.element('previewTitle');
		previewList = Doc.element('previewList');
		
		// Step 1
		Events.addEventHandler('step-1', 'wizard-validate', rf.validateStep1);
		Events.addEventHandler('caseName', 'blur', rf.caseNameOnChange);
		Events.addEventHandler('caseName', 'keypress', rf.keyPressCheck);
		//Events.addEventHandler('caseName', 'keypress', Wizard.goNextOnTabOrEnter);
		
		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', rf.initializeStep2);
		Events.addEventHandler('step-2', 'wizard-validate', rf.validateStep2);
		Events.addEventHandler('labSentDate', 'blur', rf.labDateOnChange);
		Events.addEventHandler('labReceivedDate', 'blur', rf.labDateOnChange);
		Events.addEventHandler('bloodSentDate', 'blur', rf.bloodDateOnChange);
		Events.addEventHandler('bloodReceivedDate', 'blur', rf.bloodDateOnChange);
		Events.addEventHandler('frozenSentDate', 'blur', rf.frozenDateOnChange);
		Events.addEventHandler('frozenReceivedDate', 'blur', rf.frozenDateOnChange);
		Events.addEventHandler('forwardedSentDate', 'blur', rf.forwardedDateOnChange);

		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);		

		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', rf.submit);
		
		Doc.show('step-1');
		Doc.show('gonext');
	}

	/**
		Check case name if key pressed if Tab or Return/Enter.
	*/
	rf.keyPressCheck = function(event)
	{
		// Check case name if key pressed if Tab or Return/Enter
		if (event.keyCode == 9 || event.keyCode == 13)
		{
			rf.caseNameOnChange(event);
		}
	}
	
	/**
		Check that the case number is valid.
	*/
	rf.caseNameOnChange = function(event)
	{
		var frm = document.forms['reggie'];
		var caseName = frm.caseName.value.toUpperCase();
		caseIsValid = false;
		if (caseName == '')
		{
			Wizard.setInputStatus('caseName', 'invalid', 'Missing');
			frm.caseName.focus();
			return;
		}
		else if (!Reggie.isValidCaseName(caseName, false)) // 'C' suffix is not used
		{
			rf.validatePersonalNumber(caseName);
			if (pnrIsValid)
			{
				caseIsValid = true;
			}
			else
			{
				// Entered number is neither a valid case name or personal number
				// Try to give as appropriate error message as possible
				if (caseName.length < 7 || pnrErrorMsg == null || pnrErrorMsg == '')
				{
					// The entered number is probably a faulty case name
					Wizard.setInputStatus('caseName', 'invalid', 'Only 7-digits name is allowed .');
				}
				else
				{
					Wizard.setInputStatus('caseName', 'invalid', pnrErrorMsg);
				}
				frm.caseName.focus();
				return;
			}
		}

		frm.caseName.value = caseName;

		// Check that the patient item is of type "Retract" or "RetroNo"		
		var url = '../RetractionFollowUpForm.servlet?ID='+App.getSessionId();
		url += '&cmd=GetCaseInfo&caseName=' + encodeURIComponent(caseName);
		Wizard.asyncJsonRequest(url, rf.onCaseInfoLoaded);
	}

	rf.onCaseInfoLoaded = function(response)
	{
		var frm = document.forms['reggie'];
		
		// Get biosource information from the AJAX response
		caseInfo = response.caseInfo;
		patientInfo = response.patient;

		if (patientInfo)
		{
			var patientName = patientInfo.name;
			var retractStatus = patientInfo.retractStatus;
			// Check if patient name does not start with "Retract" or "RetroNo"
			if (patientName.indexOf('Retract') != 0 && patientName.indexOf('RetroNo') != 0)
			{
				Wizard.setInputStatus('caseName', 'invalid', 'Patient name \"' + patientName + '\" does not start with \"Retract\" or \"RetroNo\".');
				return;
			}
			else if (retractStatus != null && retractStatus == 'RetractionCompleted')
			{
				Wizard.setInputStatus('caseName', 'invalid', 'Patient name \"' + patientName + '\" has \"RetractStatus\" annotation value set to \"'+retractStatus+'\".');
				return;
			}
			else
			{
				caseIsValid = true;
				Wizard.setInputStatus('caseName', 'valid');
				Wizard.goNext(true);
				return;
			}
		}
	}
	
	/**
		Check that the case number is valid.
	*/
	rf.validateStep1 = function(event)
	{
		if (event.detail.auto)
		{
			if (!caseIsValid) 
			{
				event.preventDefault();
			}
		}
		else
		{
			if (!caseIsValid) event.preventDefault();
		}
	}

	rf.validatePersonalNumber = function(pnr)
	{
		pnrIsValid = false;
		pnrIsAcceptable = true;
		pnrErrorMsg = null;
		
		var pnrLen = pnr.length;
		if (pnrLen != 12 && pnrLen != 10)
		{
			Wizard.setInputStatus('caseName', 'warning', pnrLen < 12 ? 'Personal number too short' : 'Personal number too long');
			if (pnrLen < 12)
			{
				pnrErrorMsg = 'Personal number too short';
			}
			else
			{
				pnrErrorMsg = 'Personal number too long';
			}
			return;
		}

		if (!pnr.match(/^\d+$/))
		{
			if (!pnr.match(/^\d+[A-Za-z]$/))
			{
				return;
			}
		}
		
		var datePart = pnr.substring(0, pnrLen - 4); // Remove last four digits
		var dateFormat = pnrLen == 10 ? 'yyMMdd' : 'yyyyMMdd';
		var isValidDate = Dates.isDate(datePart, dateFormat);
		if (!isValidDate)
		{
			var dayInMonth = parseInt(datePart.substring(-2), 10); // Last two characters
			if (dayInMonth > 60)
			{
				// 'Samordningsnummer' has day-in-month + 60; check this
				dayInMonth -= 60;
				var tmpPnr = pnr.substring(0, pnrLen - 6); // First 4 or 6 digits is year+month
				if (dayInMonth < 10) tmpPnr += '0';
				tmpPnr += dayInMonth;
				isValidDate = Dates.isDate(tmpPnr, dateFormat);
			}
		}
		if (!isValidDate)
		{
			Wizard.setInputStatus('caseName', 'warning', 'Personal number: Not a valid date');
			pnrErrorMsg = 'Personal number: Not a valid date';
			return;
		}
		
		if (!pnr.match(/^\d+[A-Za-z]$/))
		{
			if (!Reggie.personalNumberControlDigitCheck(pnr.substr(pnrLen == 10 ? 0 : 2)))
			{
				Wizard.setInputStatus('caseName', 'warning', 'Personal number: Invalid control digit');
				pnrErrorMsg = 'Personal number: Invalid control digit';
				return;
			}
		}

		Wizard.setInputStatus('caseName', 'valid');
		pnrIsValid = true;
	}

	/**
		Load information about the given case.
	*/
	rf.initializeStep2 = function(event)
	{
		var frm = document.forms['reggie'];
		var caseName = frm.caseName.value;

		var url = '../RetractionFollowUpForm.servlet?ID='+App.getSessionId();
		url += '&cmd=GetCaseInfo&caseName=' + encodeURIComponent(caseName);
		
		Wizard.showLoadingAnimation('Loading case information...');
		Wizard.asyncJsonRequest(url, rf.caseInfoLoaded);
	}

	/**
		Initalize the second step based on the information we have about the case.
	*/
	rf.caseInfoLoaded = function(response)
	{
		var frm = document.forms['reggie'];
		var caseName = frm.caseName.value;
		patientInfo = response.patient;
		var caseInfo = response.caseInfo;

		// Initialize biosource retraction request and response dates
		var followUpRetract = false;
		if (patientInfo)
		{
			var patientName = patientInfo.name;
			// Check if patient name starts with "Retract" or "RetroNo"
			if (patientName.indexOf('Retract') == 0 || patientName.indexOf('RetroNo') == 0)
			{
				followUpRetract = true;
				var registrationDate = patientInfo.registrationDate;
				frm.patientName.value = patientName;
				frm.patientRegistrationDate.value = registrationDate;
				// Retraction request and response dates
				frm.labSentDate.value = patientInfo.labSentDate;
				frm.labReceivedDate.value = patientInfo.labReceivedDate;
				frm.bloodSentDate.value = patientInfo.bloodSentDate;
				frm.bloodReceivedDate.value = patientInfo.bloodReceivedDate;
				frm.frozenSentDate.value = patientInfo.frozenSentDate;
				frm.frozenReceivedDate.value = patientInfo.frozenReceivedDate;
				frm.forwardedSentDate.value = patientInfo.forwardedSentDate;
			}
		}
		frm.patientName.disabled = true;
		frm.patientRegistrationDate.disabled = true;

		Wizard.setCurrentStep(2);

		Doc.show('step-1');
		if (followUpRetract)
		{
			Doc.hide('gonext');
			Doc.show('gocancel');
			Doc.show('goregister');
		}
	}

	rf.labDateOnChange = function()
	{
		Wizard.setInputStatus('labDate');
		var frm = document.forms['reggie'];		

		labSentDateIsValid = false;
		var labSentDate = frm.labSentDate.value;
		labSentDate = rf.dateCheck(labSentDate, 'lab sent', 'labDate');		
		if (labSentDate != null)
		{
			frm.labSentDate.value = labSentDate;
			labSentDateIsValid = true;
		}

		labReceivedDateIsValid = false;
		var labReceivedDate = frm.labReceivedDate.value;
		labReceivedDate = rf.dateCheck(labReceivedDate, 'lab received', 'labDate');		
		if (labReceivedDate != null)
		{
			frm.labReceivedDate.value = labReceivedDate;
			labReceivedDateIsValid = true;
		}

		if (labSentDateIsValid && labReceivedDateIsValid)
		{
			var result = rf.causalityDateCheck(labSentDate, 'lab sent', labReceivedDate, 'lab received', 'labDate');
			if (result != null)
			{
				Wizard.setInputStatus('labDate', 'valid');
			}
		}
	}

	rf.bloodDateOnChange = function()
	{
		Wizard.setInputStatus('bloodDate');
		var frm = document.forms['reggie'];		

		bloodSentDateIsValid = false;
		var bloodSentDate = frm.bloodSentDate.value;
		bloodSentDate = rf.dateCheck(bloodSentDate, 'blood sent', 'bloodDate');		
		if (bloodSentDate != null)
		{
			frm.bloodSentDate.value = bloodSentDate;
			bloodSentDateIsValid = true;
		}

		bloodReceivedDateIsValid = false;
		var bloodReceivedDate = frm.bloodReceivedDate.value;
		bloodReceivedDate = rf.dateCheck(bloodReceivedDate, 'blood received', 'bloodDate');		
		if (bloodReceivedDate != null)
		{
			frm.bloodReceivedDate.value = bloodReceivedDate;
			bloodReceivedDateIsValid = true;
		}

		if (bloodSentDateIsValid && bloodReceivedDateIsValid)
		{
			var result = rf.causalityDateCheck(bloodSentDate, 'blood sent', bloodReceivedDate, 'blood received', 'bloodDate');
			if (result != null)
			{
				Wizard.setInputStatus('bloodDate', 'valid');
			}
		}
	}

	rf.frozenDateOnChange = function()
	{
		Wizard.setInputStatus('frozenDate');
		var frm = document.forms['reggie'];		

		frozenSentDateIsValid = false;
		var frozenSentDate = frm.frozenSentDate.value;
		frozenSentDate = rf.dateCheck(frozenSentDate, 'frozen sent', 'frozenDate');		
		if (frozenSentDate != null)
		{
			frm.frozenSentDate.value = frozenSentDate;
			frozenSentDateIsValid = true;
		}

		frozenReceivedDateIsValid = false;
		var frozenReceivedDate = frm.frozenReceivedDate.value;
		frozenReceivedDate = rf.dateCheck(frozenReceivedDate, 'frozen received', 'frozenDate');		
		if (frozenReceivedDate != null)
		{
			frm.frozenReceivedDate.value = frozenReceivedDate;
			frozenReceivedDateIsValid = true;
		}

		if (frozenSentDateIsValid && frozenReceivedDateIsValid)
		{
			var result = rf.causalityDateCheck(frozenSentDate, 'frozen sent', frozenReceivedDate, 'frozen received', 'frozenDate');
			if (result != null)
			{
				Wizard.setInputStatus('frozenDate', 'valid');
			}
		}
	}

	rf.forwardedDateOnChange = function()
	{
		Wizard.setInputStatus('forwardedDate');
		var frm = document.forms['reggie'];		

		forwardedSentDateIsValid = false;
		var forwardedSentDate = frm.forwardedSentDate.value;
		forwardedSentDate = rf.dateCheck(forwardedSentDate, 'forwarded sent', 'forwardedDate');		
		if (forwardedSentDate != null)
		{
			frm.forwardedSentDate.value = forwardedSentDate;
			forwardedSentDateIsValid = true;
		}

		if (forwardedSentDateIsValid)
		{
			Wizard.setInputStatus('forwardedDate', 'valid');
		}
	}

	rf.dateCheck = function(dateValue, dateName, dateMessageKey)
	{
		if (dateValue != '')
		{
			// Auto-fill the date if it's only given with 4(MMdd) or 6(yyMMdd) digits.
			dateValue = Reggie.autoFillDate(dateValue, true);

			var d = Dates.parseString(dateValue, 'yyyyMMdd');
			if (d == null)
			{
				Wizard.setInputStatus(dateMessageKey, 'invalid', 'Not a valid '+dateName+' date');
				return null;
			}
			
			var today = new Date();
			if (d.getTime() > today.getTime())
			{
				Wizard.setInputStatus(dateMessageKey, 'invalid', 'Future '+dateName+' date is not valid');
				return null;
			}			
		}
		return dateValue;
	}

	rf.causalityDateCheck = function(date1, dateName1, date2, dateName2, dateMessageKey)
	{
		if (date1 != null && date1 != '' && date2 != null && date2 != '')
		{
			// Auto-fill the date if it's only given with 4(MMdd) or 6(yyMMdd) digits.
			date1 = Reggie.autoFillDate(date1, true);
			date2 = Reggie.autoFillDate(date2, true);

			var d1 = Dates.parseString(date1, 'yyyyMMdd');
			if (d1 == null)
			{
				Wizard.setInputStatus(dateMessageKey, 'invalid', 'Not a valid '+dateName1+' date');
				return null;
			}
			
			var d2 = Dates.parseString(date2, 'yyyyMMdd');
			if (d2 == null)
			{
				Wizard.setInputStatus(dateMessageKey, 'invalid', 'Not a valid '+dateName2+' date');
				return null;
			}
			
			if (d1.getTime() > d2.getTime())
			{
				Wizard.setInputStatus(dateMessageKey, 'invalid', 'Causality violation: '+dateName1+' date is later than '+dateName2+' date');
				return null;
			}	
		}
		return date1;
	}

	rf.validateStep2 = function(event)
	{
		// Get up-to-date values for date validation variables
		rf.labDateOnChange();
		rf.bloodDateOnChange();
		rf.frozenDateOnChange();
		rf.forwardedDateOnChange();

		// Step 2 is valid if all date values are valid ('null' value is allowed)
		var datesAreValid = false;
		if (labSentDateIsValid &&
			labReceivedDateIsValid &&
			bloodSentDateIsValid &&
			bloodReceivedDateIsValid &&
			frozenSentDateIsValid &&
			frozenReceivedDateIsValid &&
			forwardedSentDateIsValid)
		{
			datesAreValid = true;
		}
		
		if (!datesAreValid) event.preventDefault();
	}
	
	// Style the message as a warning message
	rf.asWarning = function(message)
	{
		message = '<span class="warning">' + message + '</span>';
		return message;
	}

	// Generate a 'Missing' warning if the value is missing
	rf.warnIfMissing = function(value)
	{
		var message = value ? value : rf.asWarning('Missing');
		return message;
	}
	
	rf.goPrint = function()
	{
		var caseName = Data.get('page-data', 'case-name');
		var printNote = '<b>Note!</b> For better printing set page orientation to <i>portrait</i>.<br>';
		printNote += ' You may have to <i>scale down</i> to fit everything on the width of the page.';
		Reggie.openPrintWindow('all-content', 'Case summary - ' + Strings.encodeTags(caseName), 'portrait', printNote, '../', 'case_summary.css');
	}

	rf.submit = function()
	{
		var frm = document.forms['reggie'];

		var retractionInfo = {};
		retractionInfo.caseName = frm.caseName.value;
		retractionInfo.patientName = frm.patientName.value;
		retractionInfo.labSentDate = frm.labSentDate.value;
		retractionInfo.labReceivedDate = frm.labReceivedDate.value;
		retractionInfo.bloodSentDate = frm.bloodSentDate.value;
		retractionInfo.bloodReceivedDate = frm.bloodReceivedDate.value;
		retractionInfo.frozenSentDate = frm.frozenSentDate.value;
		retractionInfo.frozenReceivedDate = frm.frozenReceivedDate.value;
		retractionInfo.forwardedSentDate = frm.forwardedSentDate.value;

		var submitInfo = {};
		submitInfo.retractionInfo = retractionInfo;
		
		var url = '../RetractionFollowUpForm.servlet?ID=' + App.getSessionId();
		url += '&cmd=RegisterRetraction';

		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, rf.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	rf.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}

	return rf;
}();

Doc.onLoad(RetractionFollowUpForm.initPage);

